/*
 * Sia E(u) gli archi incidenti al nodo u
 * L'algoritmo ordina i nodi in B in modo greedy, scegliendo ogni volta
 * il nodo u che minimizza \sum{r \in R} count_crossings(E(u), E(r)),
 * dove R è l'insieme dei nodi non ancora usati, e la funzione
 * count_crossings(E1, E2) conta il numero di incroci tra i due insiemi di
 * archi E1, E2.
 *
 * Complessità O(m^2 + n^3)
 */
#include "nonna.h"
#include <algorithm>
#include <bits/stdc++.h>
#include <bitset>
#include <climits>
#include <iterator>
#include <numeric>
#include <random>
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream& in(cin);
ostream& out(cout);
#endif

constexpr int NONE = -1;
constexpr int MAX_N_DP = 10;
constexpr int SEED = 420;

random_device rd;
mt19937 rng(SEED);

using node_t = int;
using nodevec_t = vector<node_t>;
using layer_t = vector<nodevec_t>;
using ivec_t = vector<long>;
using imatrix_t = vector<ivec_t>;

/** Assume v is sorted **/
int median(nodevec_t& v) {
    switch (v.size()) {
    case 0:
        return 0;
    default:
        return v[v.size() / 2];
    }
}

double mean(const nodevec_t& v) {
    switch (v.size()) {
    case 0:
        return 0;
    case 1:
        return v[0];
    default:
        int sum = std::accumulate(v.begin(), v.end(), 0);
        return 1.0L * sum / v.size();
    }
}

void print_sol(const nodevec_t& order, const long crossings) {
    out << crossings << endl;
    for (node_t a : order) {
        out << a << " ";
    }
    out << "\n***" << endl;
}

class FenwickTree {
  private:
    ivec_t bit; // binary indexed tree
    int n;

  public:
    FenwickTree(int n) : n(n), bit(n + 1, 0) {}

    void reset() { bit.assign(n + 1, 0); }

    long sum(int r) {
        assert(bit.size() == n + 1);
        assert(r <= n);
        long ret = 0;
        for (; r > 0; r -= r & -r) {
            assert(r < bit.size());
            ret += bit[r];
        }
        return ret;
    }

    void add(int idx, long delta) {
        assert(bit.size() == n + 1);
        for (; idx <= n; idx += idx & -idx) {
            assert(idx >= 0);
            assert(idx < bit.size());
            bit[idx] += delta;
        }
    }
};

class Solver {
  private:
    int n_a, n_b;
    long min_improvement;
    layer_t a_layer;
    imatrix_t C;
    FenwickTree ft;

  public:
    Solver(int n_a, int n_b, int m)
        : n_a(n_a), n_b(n_b), a_layer(n_a), ft(n_b){};

    void add_edge(node_t a, node_t b) { a_layer[a].push_back(b); }

    inline const nodevec_t& neigh(node_t u) { return a_layer[u]; }

    inline bool use_matrix() {
        return n_a < 1500;
        // return true;
    }

    /**
     * Returns the number of edges incident to u and v that would cross if u
     * comes before v
     */
    long pair_crossings(node_t u, node_t v) {
        if (use_matrix()) {
            return C[u][v];
        }
        return _pair_crossings(u, v);
    }

    long _pair_crossings(node_t u, node_t v) {
        long count = 0;
        for (auto itu = a_layer[u].rbegin(); itu != a_layer[u].rend(); ++itu) {
            for (auto itv = a_layer[v].begin();
                 itv != a_layer[v].end() && *itv < *itu; ++itv) {
                count++;
            }
        }
        return count;
    }

    /**
     * Computes the crossings matrix C where
     * C[u][v] is the number of edge crossings between edges incident to u,v in
     * layer_a, assuming u is to the left of v.
     */
    void init_crossings_matrix() {
        if (!use_matrix()) {
            return;
        }

        C.assign(n_a, vector<long>(n_a, 0));
        for (int ua = 0; ua < n_a; ua++) {
            for (int va = ua + 1; va < n_a; va++) {
                C[ua][va] = _pair_crossings(ua, va);
                C[va][ua] = _pair_crossings(va, ua);
            }
        }
    }

    long init(nodevec_t& order) {
        for (node_t a = 0; a < n_a; a++) {
            sort(a_layer[a].begin(), a_layer[a].end());
        }
        init_crossings_matrix();
        order.resize(n_a);
        iota(order.begin(), order.end(), 0);

        return compute_crossings(order.begin(), order.end());
    }

    long compute_crossings(const nodevec_t::iterator& begin,
                           const nodevec_t::iterator& end) {
        long ncrossings = 0;
        ft.reset();
        reverse_iterator<nodevec_t::iterator> rbegin(end);
        reverse_iterator<nodevec_t::iterator> rend(begin);
        for (auto ita = rbegin; ita != rend; ++ita) {
            assert(*ita >= 0);
            assert(*ita < n_a);
            const nodevec_t& adj = neigh(*ita);
            for (auto itb = adj.rbegin(); itb != adj.rend(); ++itb) {
                assert(*itb >= 0);
                assert(*itb < n_b);
                node_t b = *itb;
                ncrossings += ft.sum(b);
                ft.add(b + 1, 1);
            }
        }
        /* for (nodevec_t::iterator it1 = begin; it1 != end; ++it1) { */
        /*     for (nodevec_t::iterator it2 = it1 + 1; it2 != end; ++it2) { */
        /*         ncrossings += pair_crossings(*it1, *it2); */
        /*     } */
        /* } */

        return ncrossings;
    }

    template <typename T>
    void sort_by(nodevec_t& order, const vector<T>& metric) {
        sort(order.begin(), order.end(),
             [&metric](const node_t& a, const node_t& b) {
                 return metric[a] < metric[b];
             });
    }

    void sort_by_avg_neigh(nodevec_t& order) {
        vector<double> avg_neig(n_a, 0);
        for (node_t a = 0; a < n_a; a++) {
            avg_neig[a] = mean(neigh(a));
        }
        sort_by(order, avg_neig);
    }

    void sort_by_cross_matrix(nodevec_t& order) {
        sort(order.begin(), order.end(),
             [this](const node_t& a, const node_t& b) {
                 return this->pair_crossings(a, b) < pair_crossings(b, a);
             });
    }

    void sort_by_med_neigh(nodevec_t& order) {
        vector<int> med_neig(n_a, 0);
        for (node_t a = 0; a < n_a; a++) {
            med_neig[a] = median(a_layer[a]);
        }
        sort_by(order, med_neig);
    }

    void solve_sort(nodevec_t& order, long& crossings) {
        nodevec_t tmp_order = order;
        long tmp_crossings = crossings;

        /**
         * Sort by average
         */
        sort_by_avg_neigh(tmp_order);
        tmp_crossings = compute_crossings(tmp_order.begin(), tmp_order.end());

        if (tmp_crossings < crossings) {
            crossings = tmp_crossings;
            order = tmp_order;
            print_sol(order, crossings);
        }

        /**
         * Sort by median
         */
        sort_by_med_neigh(tmp_order);
        tmp_crossings = compute_crossings(tmp_order.begin(), tmp_order.end());
        if (tmp_crossings < crossings) {
            crossings = tmp_crossings;
            order = tmp_order;
            print_sol(order, crossings);
        }

        /**
         * Sort by cross matrix
         */
        sort_by_cross_matrix(tmp_order);
        tmp_crossings = compute_crossings(tmp_order.begin(), tmp_order.end());
        if (tmp_crossings < crossings) {
            crossings = tmp_crossings;
            order = tmp_order;
            print_sol(order, crossings);
        }
    }

    long solve_blocks_dp_rec(const nodevec_t::iterator& begin,
                             const nodevec_t::iterator& end,
                             bitset<MAX_N_DP>& unchosen, ivec_t& DP) {
        /* cerr << "Unchosen: " << unchosen.to_string() << endl; */

        if (unchosen.none()) {
            DP[unchosen.to_ulong()] = 0;
        } else if (DP[unchosen.to_ulong()] == NONE) {
            /* cerr << "\tis none, computing it" << endl; */
            long res = LONG_MAX;
            for (auto itu = begin; itu != end; ++itu) {
                size_t idxu = itu - begin;
                /* cerr << "\t\tTrying node " << *itu << endl; */
                if (unchosen[idxu]) {
                    unchosen.reset(idxu);

                    // by inserting node u, compute all crossings between u and
                    // all other unchosen nodes v
                    long next_cost = 0;
                    for (auto itv = begin; itv != end; ++itv) {
                        size_t idxv = itv - begin;
                        /* cerr << "\t\t\tCompare with " << *itv << endl; */

                        if (unchosen[idxv]) {
                            next_cost += pair_crossings(*itu, *itv);
                        }
                    }
                    /* cerr << "\t\tcost: " << next_cost << endl; */

                    res = min(res, next_cost + solve_blocks_dp_rec(
                                                   begin, end, unchosen, DP));
                    unchosen.set(idxu);
                }
            }
            /* cerr << "res: " << res << " unchosen: " << unchosen.to_string()
             */
            /*      << endl; */
            DP[unchosen.to_ulong()] = res;
        }
        return DP[unchosen.to_ulong()];
    }

    void restore_optimal_order_dp(const nodevec_t::iterator& begin,
                                  const nodevec_t::iterator& end,
                                  const ivec_t& DP) {
        bitset<MAX_N_DP> unchosen;
        nodevec_t orig_order(begin, end);
        size_t vec_size = end - begin;
        unchosen = (1 << vec_size) - 1;
        for (auto it = begin; it != end; ++it) {
            /* Find next node, i.e., the node u s.t. u is unchosen and
             * DP[unchosen] == DP[unchosen - u] + cost(u) */
            nodevec_t::iterator itu;
            long opt_cost = DP[unchosen.to_ulong()];
            /* cerr << "unchosen: " << unchosen.to_string() */
            /*      << ", cost: " << opt_cost << endl; */

            for (itu = orig_order.begin(); itu != orig_order.end(); ++itu) {
                node_t u = *itu;
                size_t idxu = itu - orig_order.begin();
                if (unchosen[idxu]) {
                    unchosen.reset(idxu);
                    long costu = DP[unchosen.to_ulong()];
                    for (nodevec_t::iterator itv = orig_order.begin();
                         itv != orig_order.end(); ++itv) {
                        node_t v = *itv;
                        size_t idxv = itv - orig_order.begin();
                        if (unchosen[idxv]) {
                            /* cerr << "\twould cross with " << *itv << endl; */
                            costu += pair_crossings(u, v);
                        }
                    }
                    /* cerr << "Cost by choosing " << u << ": " << costu <<
                     * endl; */
                    if (opt_cost == costu) {
                        break;
                    } else {
                        unchosen.set(idxu);
                    }
                }
            }
            /* cerr << "Best u: " << *itu << endl; */
            *it = *itu;
        }
    }

    long solve_blocks_dp(nodevec_t& order, long crossings) {
        bitset<MAX_N_DP> unchosen;
        auto x = order.begin();
        ivec_t DP;
        for (size_t step = MAX_N_DP; step > 0; step -= 2) {
            for (nodevec_t::iterator it = order.begin(); it < order.end();
                 it += step) {
                nodevec_t::iterator end = min(order.end(), it + MAX_N_DP);
                size_t range = end - it;
                long curr_cost = compute_crossings(it, end);
                unchosen = (1 << range) - 1;
                DP.assign(1 + unchosen.to_ulong(), NONE);
                long new_cost = solve_blocks_dp_rec(it, end, unchosen, DP);
                if (new_cost < curr_cost) {
                    restore_optimal_order_dp(it, end, DP);
                    crossings += new_cost - curr_cost;
                    print_sol(order, crossings);
                }
            }
        }
        return crossings;
    }
};

int main() {
    int n_a, n_b, m;
    in >> n_a >> n_b >> m;
    Solver solver(n_a, n_b, m);
    for (int i = 0; i < m; i++) {
        node_t a, b;
        in >> a >> b;
        b -= n_a;
        solver.add_edge(a, b);
    }
    /* solver.print_graph(); */
    nodevec_t best_order(n_a);
    long best_crossings = solver.init(best_order);
    print_sol(best_order, best_crossings);

    solver.solve_sort(best_order, best_crossings);
    solver.solve_blocks_dp(best_order, best_crossings);

    return 0;
}
